package socialnetworkservicelogic

import (
	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/domain/repository"
	mocklib "app-cloudep-tweeting-service/internal/mock/lib"
	mockRepo "app-cloudep-tweeting-service/internal/mock/repository"
	"app-cloudep-tweeting-service/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestGetFolloweeLogic_GetFollowee(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockSocialNetworkRepository := mockRepo.NewMockSocialNetworkRepository(ctrl)
	mockValidate := mocklib.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		SocialNetworkRepository: mockSocialNetworkRepository,
		Validate:                mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *tweeting.FollowReq
		prepare   func()
		expectErr bool
		wantResp  *tweeting.FollowResp
	}{
		{
			name: "成功獲取我跟隨的名單",
			input: &tweeting.FollowReq{
				Uid:       "12345",
				PageSize:  10,
				PageIndex: 1,
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬 GetFollowee 返回正確的結果
				mockSocialNetworkRepository.EXPECT().GetFollowee(gomock.Any(), repository.FollowReq{
					UID:       "12345",
					PageSize:  10,
					PageIndex: 1,
				}).Return(repository.FollowResp{
					UIDs:  []string{"user1", "user2"},
					Total: 2,
				}, nil).Times(1)
			},
			expectErr: false,
			wantResp: &tweeting.FollowResp{
				Uid: []string{"user1", "user2"},
				Page: &tweeting.Pager{
					Total: 2,
					Index: 1,
					Size:  10,
				},
			},
		},
		{
			name: "驗證失敗",
			input: &tweeting.FollowReq{
				Uid:       "",
				PageSize:  10,
				PageIndex: 1,
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("validation failed")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
		{
			name: "獲取我跟隨的名單失敗",
			input: &tweeting.FollowReq{
				Uid:       "12345",
				PageSize:  10,
				PageIndex: 1,
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬 GetFollowee 返回錯誤
				mockSocialNetworkRepository.EXPECT().GetFollowee(gomock.Any(), repository.FollowReq{
					UID:       "12345",
					PageSize:  10,
					PageIndex: 1,
				}).Return(repository.FollowResp{}, errors.New("repository error")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
	}

	// 執行測試
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 GetFolloweeLogic
			logic := GetFolloweeLogic{
				svcCtx: svcCtx,
				ctx:    context.TODO(),
			}

			// 執行 GetFollowee
			got, err := logic.GetFollowee(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, got)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.wantResp, got)
			}
		})
	}
}
