package socialnetworkservicelogic

import (
	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	mocklib "app-cloudep-tweeting-service/internal/mock/lib"
	mockRepo "app-cloudep-tweeting-service/internal/mock/repository"
	"app-cloudep-tweeting-service/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestGetFollowerCountLogic_GetFollowerCount(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockSocialNetworkRepository := mockRepo.NewMockSocialNetworkRepository(ctrl)
	mockValidate := mocklib.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		SocialNetworkRepository: mockSocialNetworkRepository,
		Validate:                mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *tweeting.FollowCountReq
		prepare   func()
		expectErr bool
		wantResp  *tweeting.FollowCountResp
	}{
		{
			name: "成功獲取跟隨者數量",
			input: &tweeting.FollowCountReq{
				Uid: "12345",
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬 GetFollowerCount 返回正確的結果
				mockSocialNetworkRepository.EXPECT().GetFollowerCount(gomock.Any(), "12345").Return(int64(10), nil).Times(1)
			},
			expectErr: false,
			wantResp: &tweeting.FollowCountResp{
				Uid:   "12345",
				Total: 10,
			},
		},
		{
			name: "驗證失敗",
			input: &tweeting.FollowCountReq{
				Uid: "",
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("validation failed")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
		{
			name: "獲取跟隨者數量失敗",
			input: &tweeting.FollowCountReq{
				Uid: "12345",
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬 GetFollowerCount 返回錯誤
				mockSocialNetworkRepository.EXPECT().GetFollowerCount(gomock.Any(), "12345").Return(int64(0), errors.New("repository error")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
	}

	// 執行測試
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 GetFollowerCountLogic
			logic := GetFollowerCountLogic{
				svcCtx: svcCtx,
				ctx:    context.TODO(),
			}

			// 執行 GetFollowerCount
			got, err := logic.GetFollowerCount(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, got)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.wantResp, got)
			}
		})
	}
}
