package timelineservicelogic

import (
	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/domain/repository"
	mocklib "app-cloudep-tweeting-service/internal/mock/lib"
	mockRepo "app-cloudep-tweeting-service/internal/mock/repository"
	"app-cloudep-tweeting-service/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestFetchTimelineLogic_FetchTimeline(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockTimelineRepo := mockRepo.NewMockTimelineRepository(ctrl)
	mockValidate := mocklib.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		TimelineRepo: mockTimelineRepo,
		Validate:     mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *tweeting.GetTimelineReq
		prepare   func()
		expectErr bool
		wantResp  *tweeting.FetchTimelineResponse
	}{
		{
			name: "成功獲取動態時報",
			input: &tweeting.GetTimelineReq{
				Uid:       "user123",
				PageSize:  10,
				PageIndex: 1,
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬成功獲取動態時報
				mockTimelineRepo.EXPECT().FetchTimeline(gomock.Any(), repository.FetchTimelineRequest{
					UID:       "user123",
					PageSize:  10,
					PageIndex: 1,
				}).Return(repository.FetchTimelineResponse{
					Items: []repository.TimelineItem{
						{PostID: "post1", Score: 100},
						{PostID: "post2", Score: 200},
					},
					Page: tweeting.Pager{
						Total: 2,
						Size:  10,
						Index: 1,
					},
				}, nil).Times(1)
			},
			expectErr: false,
			wantResp: &tweeting.FetchTimelineResponse{
				Posts: []*tweeting.FetchTimelineItem{
					{PostId: "post1", Score: 100},
					{PostId: "post2", Score: 200},
				},
				Page: &tweeting.Pager{
					Total: 2,
					Index: 1,
					Size:  10,
				},
			},
		},
		{
			name: "驗證失敗",
			input: &tweeting.GetTimelineReq{
				Uid:       "",
				PageSize:  10,
				PageIndex: 1,
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("validation failed")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
		{
			name: "獲取動態時報失敗",
			input: &tweeting.GetTimelineReq{
				Uid:       "user123",
				PageSize:  10,
				PageIndex: 1,
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬獲取動態時報失敗
				mockTimelineRepo.EXPECT().FetchTimeline(gomock.Any(), repository.FetchTimelineRequest{
					UID:       "user123",
					PageSize:  10,
					PageIndex: 1,
				}).Return(repository.FetchTimelineResponse{}, errors.New("repository error")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
	}

	// 執行測試
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 FetchTimelineLogic
			logic := FetchTimelineLogic{
				svcCtx: svcCtx,
				ctx:    context.TODO(),
			}

			// 執行 FetchTimeline
			got, err := logic.FetchTimeline(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, got)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.wantResp, got)
			}
		})
	}
}
