package timelineservicelogic

import (
	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	mocklib "app-cloudep-tweeting-service/internal/mock/lib"
	mockRepo "app-cloudep-tweeting-service/internal/mock/repository"
	"app-cloudep-tweeting-service/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestSetNoMoreDataFlagLogic_SetNoMoreDataFlag(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockTimelineRepo := mockRepo.NewMockTimelineRepository(ctrl)
	mockValidate := mocklib.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		TimelineRepo: mockTimelineRepo,
		Validate:     mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *tweeting.DoNoMoreDataReq
		prepare   func()
		expectErr bool
		wantResp  *tweeting.OKResp
	}{
		{
			name: "成功標記時間線已完整",
			input: &tweeting.DoNoMoreDataReq{
				Uid: "user123",
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬成功標記時間線狀態
				mockTimelineRepo.EXPECT().SetNoMoreDataFlag(gomock.Any(), "user123").Return(nil).Times(1)
			},
			expectErr: false,
			wantResp:  &tweeting.OKResp{},
		},
		{
			name: "驗證失敗",
			input: &tweeting.DoNoMoreDataReq{
				Uid: "",
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("validation failed")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
		{
			name: "標記時間線狀態失敗",
			input: &tweeting.DoNoMoreDataReq{
				Uid: "user123",
			},
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬標記時間線狀態失敗
				mockTimelineRepo.EXPECT().SetNoMoreDataFlag(gomock.Any(), "user123").Return(errors.New("repository error")).Times(1)
			},
			expectErr: true,
			wantResp:  nil,
		},
	}

	// 執行測試
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 SetNoMoreDataFlagLogic
			logic := SetNoMoreDataFlagLogic{
				svcCtx: svcCtx,
				ctx:    context.TODO(),
			}

			// 執行 SetNoMoreDataFlag
			got, err := logic.SetNoMoreDataFlag(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, got)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.wantResp, got)
			}
		})
	}
}
